function WebpackPluginInnerScript(options) {
  this.options = options || {};
}

WebpackPluginInnerScript.prototype.apply = function(compiler) {
  var self = this;
  if (compiler.hooks) {
    compiler.hooks.compilation.tap('HtmlWebpackReplace', function(compilation) {
      compilation.hooks.htmlWebpackPluginAlterAssetTags.tapAsync(
        'HtmlWebpackReplace',
        self.innerScript.bind(self)
      );
    });

    compiler.hooks.compilation.tap('HtmlWebpackRemoveHead', function(compilation) {
      compilation.hooks.htmlWebpackPluginAfterHtmlProcessing.tapAsync(
        'HtmlWebpackRemoveHead',
        self.innerScript.bind(self)
      );
    });
  } else {
    compiler.plugin('compilation', compilation => {
      compilation.plugin(
        'html-webpack-plugin-alter-asset-tags',
        self.innerScript.bind(self)
      );
    });

    compiler.plugin('compilation', compilation => {
      compilation.plugin(
        'html-webpack-plugin-after-html-processing',
        self.removeHead.bind(self)
      );
    });
  }
};

WebpackPluginInnerScript.prototype.innerScript = function(
  htmlPluginData,
  callback
) {
  if (this.options.hasOwnProperty('innerScript') && this.options.innerScript) {
    ['head', 'body'].map(position => {
      for (var i = 0; i < htmlPluginData[position].length; i++) {
        const chunk = htmlPluginData[position][i];
        let isIgnore = false;
        const url = chunk.attributes.src;
        const template = htmlPluginData.outputName;
        if (
          this.options.hasOwnProperty('ignore') &&
          this.options.ignore.length > 0
        ) {
          this.options.ignore.map(item => {
            if (istype(item, 'RegExp') && item.test(url)) {
              isIgnore = true;
            }
          });
        }
        if (isIgnore) {
          break;
        }

        let isInclude = false;
        if (
          this.options.hasOwnProperty('include') &&
          this.options.include.length > 0
        ) {
          this.options.include.map(item => {
            if (istype(item, 'RegExp') && template.match(item)) {
              isInclude = true;
            }
          });
        }
        if (isInclude && chunk.attributes.type === 'text/javascript') {
          chunk.innerHTML = `
              var oHead = document.querySelector('body'),
                  oScript = document.createElement('script');
              oScript.type = "text/javascript";
              oScript.src = "${url}";
              oHead.appendChild(oScript);
              `;
          // 针对插件做的处理
          if (url.match(/plugin-skins/)) {
            let skinName;
            if (process.platform.match(/win32/g)) {
              skinName = htmlPluginData.outputName.match(/plugin\\skins\\(.*?)\\entry/)[1]
            } else {
              skinName = htmlPluginData.outputName.match(/plugin\/skins\/(.*?)\/entry/)[1]
            }
            
            const errorMessage1 = `\'找不到window.TA.pluginSkin[\"${skinName}\"]，同页面出多个插件会发生冲突，请将皮肤代码中的window.TA.pluginSkin改为window.TA.pluginSkin[\"${skinName}\"]\'`;
            const errorMessage2 = `\'找不到window.TA.pluginSkin或window.TA.pluginSkin[\"${skinName}\"]，插件将无法出现，请检查皮肤代码\'`;
            chunk.innerHTML += `
              window.TA.pluginSkin = window.TA.pluginSkin || {};
              oScript.onload = function() {
                if (window.TA.pluginSkin && window.TA.pluginSkin[\"${skinName}\"]) {
                  window.TA.pluginSkin[\"${skinName}\"].init();
                } else if (window.TA.pluginSkin) {
                  ${process.env.NODE_ENV === 'production' ? `console.error(${errorMessage1})` : `alert(${errorMessage1})`}
                } else {
                  ${process.env.NODE_ENV === 'production' ? `console.error(${errorMessage2})` : `alert(${errorMessage2})`}
                }
              }
            `
          }
          chunk.attributes = {
            type: 'text/javascript'
          };
        }
      }
    });
  }
  callback(null, htmlPluginData);
};

WebpackPluginInnerScript.prototype.removeHead = function(
  htmlPluginData,
  callback
) {
  if (this.options.hasOwnProperty('innerScript') && this.options.innerScript) {
    const template = htmlPluginData.outputName;
    let isInclude = false;
    if (
      this.options.hasOwnProperty('include') &&
      this.options.include.length > 0
    ) {
      this.options.include.map(item => {
        if (istype(item, 'RegExp') && template.match(item)) {
          isInclude = true;
        }
      });
    }
    if (isInclude) {
      const result = htmlPluginData.html.match(/<head>(.*?)<\/head>/);
      if (result) {
        htmlPluginData.html = htmlPluginData.html.replace(result[0], result[1]);
      }
    }
  }
  callback(null, htmlPluginData);
};

function istype(o, type) {
  return (
    Object.prototype.toString.call(o) === '[object ' + (type || 'Object') + ']'
  );
}

module.exports = WebpackPluginInnerScript;
